/* moz_bio.c    - mozilla BIO interface
 *
 * This is the interface glue that provides the SSLeay BIO which simply
 * uses the hooked-in NSPR layer as the actual communication interface.
 *
 * http://mozilla-crypto.ssleay.org
 *
 */

#include <stdio.h>
#include <errno.h>
#include "moz_eay.h"

/* I need to rework the send/recv timeout so that it is a cumulative total,
 * which is reset only by the upper read/write/send/recv being re-done,
 * this way the multiple read/write's of the SSL protocol will be added
 * into one 
 */


/* Not really a filter or an endpoint.... */
#define BIO_TYPE_MOZILLA	(20|BIO_TYPE_DESCRIPTOR)

static int mozilla_write();
static int mozilla_read();
static long mozilla_ctrl();
static int mozilla_new();
static int mozilla_free();

typedef struct mozilla_glue_st
	{
	PRFileDesc *me;
	PRIntervalTime timeout;
	int use_timeout;
	} MOZILLA_GLUE;

static BIO_METHOD methods_mozillap=
	{
	BIO_TYPE_MOZILLA,
	"SSLeay to mozilla PRFileDesc glue",
	mozilla_write,
	mozilla_read,
	NULL, /* puts */
	NULL, /* gets, */
	mozilla_ctrl,
	mozilla_new,
	mozilla_free,
	};

BIO_METHOD *BIO_f_mozilla()
	{
	return(&methods_mozillap);
	}

static int mozilla_new(bi)
BIO *bi;
	{
	MOZILLA_GLUE *mg;

	mg=(MOZILLA_GLUE *)Malloc(sizeof(MOZILLA_GLUE));
	if (mg == NULL) return(0);
	mg->me=NULL;
	mg->timeout=0;
	mg->use_timeout=0;

	bi->init=0;
	bi->num=0;
	bi->ptr=(char *)mg;
	bi->flags=0;
	return(1);
	}

static int mozilla_free(a)
BIO *a;
	{
	if (a == NULL) return(0);
	if (a->shutdown)
		{
		if (a->ptr != NULL)
			{
			Free(a->ptr);
			a->ptr=NULL;
			}
		a->init=0;
		a->flags=0;
		}
	return(1);
	}
	
static int mozilla_read(b,out,outl)
BIO *b;
char *out;
int outl;
	{
	MOZILLA_GLUE *mg;
	int ret=0;

	BIO_clear_retry_flags(b);
	mg=(MOZILLA_GLUE *)b->ptr;
	if (out != NULL)
		{
		if (mg->use_timeout)
			{
#if 0
			PRPollDesc pds;
			
			mg->use_timeout=0;
			pds.fd=mg->me;
			pds.in_flags=PR_POLL_READ;
			i=PR_Poll(&pds,1,mg->timeout);
			if (i == 0)
				{
				PR_SetError(PR_IO_TIMEOUT_ERROR,0);
				ret= -1;
				goto err;
				}
#endif
			ret=mg->me->methods->recv(
				mg->me,out,outl,0,mg->timeout);
			}
		else
			ret=mg->me->methods->read(
				mg->me,out,outl);
		}
	if (dbglev)
		BIO_printf(bio_err,"mozilla_read: %d -> %d\n",outl,ret);
	if (ret < 0)
		BIO_set_retry_read(b);
	return(ret);
	}

static int mozilla_write(b,out,outl)
BIO *b;
char *out;
int outl;
	{
	MOZILLA_GLUE *mg;
	int ret=0;

	BIO_clear_retry_flags(b);
	mg=(MOZILLA_GLUE *)b->ptr;
	if (out != NULL)
		{
		if (mg->use_timeout)
			{
#if 0
			PRPollDesc pds;
			
			mg->use_timeout=0;
			pds.fd=mg->me;
			pds.in_flags=PR_POLL_WRITE;
			i=PR_Poll(&pds,1,mg->timeout);
			if (i == 0)
				{
				PR_SetError(PR_IO_TIMEOUT_ERROR,0);
				ret= -1;
				goto err;
				}
#endif
			ret=mg->me->methods->send(
				mg->me,out,outl,0,mg->timeout);
			}
		else	
			ret=mg->me->methods->write(
				mg->me,out,outl);
		}
	if (ret < 0)
		BIO_set_retry_write(b);
	if (dbglev)
		BIO_printf(bio_err,"mozilla_write: %d -> %d\n",outl,ret);
	return(ret);
	}


static long mozilla_ctrl(b,cmd,num,ptr)
BIO *b;
int cmd;
long num;
char *ptr;
	{
	long ret=1;
	MOZILLA_GLUE *mg=(MOZILLA_GLUE *)b->ptr;

	switch (cmd)
		{
	case BIO_CTRL_RESET:
		ret=0;
		break;
	case BIO_CTRL_INFO:
		ret=0;
		break;
	case BIO_C_SET_PR_FILEDESC:
		mg->me=(PRFileDesc *)ptr;
		b->init=1;
		break;
	case BIO_C_SET_PR_TIMEOUT:
		mg->timeout=(PRIntervalTime)num;
		mg->use_timeout=1;
		break;
	case BIO_C_GET_FD:
		break;
	case BIO_CTRL_GET_CLOSE:
		ret=b->shutdown;
		break;
	case BIO_CTRL_SET_CLOSE:
		b->shutdown=(int)num;
		break;
	case BIO_CTRL_PENDING:
	case BIO_CTRL_WPENDING:
		ret=0;
		break;
	case BIO_CTRL_DUP:
		/* Should fill this in */
		break;
	case BIO_CTRL_FLUSH:
		ret=1;
		break;
	default:
		ret=0;
		break;
		}
	return(ret);
	}

