/* ssleay.c    - main SSLeay interface logic for cryptozilla
 *
 *
 * http://mozilla-crypto.ssleay.org
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "moz_eay.h"

/* SSLeay stuff */
#include "e_os.h"
#include "bio.h"
#include "crypto.h"
#include "ssl.h"
#include "err.h"

/* naughty globals */
int ssl_window_id_idx=0;
SSL_CTX *ssl_ctx=NULL;
PRDescIdentity ssleay_ssl_identity=0;
BIO *debug_bio=NULL;
BIO *bio_err=NULL;

/* default debug to *OFF* ... set via env var CRYPTOZILLA_DEBUG=1 */
int dbglev=0; 

static PRLock *lock_cs[CRYPTO_NUM_LOCKS];
static int thread_data_index=0;

static void PR_CALLBACK mozilla_locking_callback(mode,type,file,line)
int mode;
int type;
char *file;
int line;
        {
        if (mode & CRYPTO_LOCK)
                {
                PR_Lock(lock_cs[type]);
                }
        else
		{
		PR_Unlock(lock_cs[type]);
		}
	}

/* This is called to use the by thread destructor to remove the
 * SSLeay error state, otherwise we will have memory leaks. 
 */
void mozilla_set_ssleay_error_cleanup()
	{
	PR_SetThreadPrivate(thread_data_index,(void *)"SSLeay");
	}

static void PR_CALLBACK mozilla_ssleay_error_distructor(data)
void *data;
	{
	ERR_remove_state(0);
	}

int SSLeay_init()
	{
	int i;
	char *p;

	/* have the usual interface */
    	bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
	debug_bio=bio_err;

	/* a little runtime switch ... as linking this takes way
	 * too long for us to want to switch things on an off
	 * all the time --tjh
	 */
	p=getenv("CRYPTOZILLA_DEBUG");
	if (p!=NULL)
	  dbglev=atoi(p);
	  
	if (dbglev)
	  BIO_printf(bio_err,"ssleay.c:SSLeay_init\n");

	/* fix */
	CRYPTO_set_mem_functions((char *(*)())PR_Malloc,
		(char *(*)())PR_Realloc,PR_Free);

	for (i=0; i<CRYPTO_NUM_LOCKS; i++)
		{
		lock_cs[i]=PR_NewLock();
		if (lock_cs[i] == NULL) goto err;
		}

	CRYPTO_set_locking_callback((void (*)(int,int,char *,int))
		mozilla_locking_callback);

	/* Thread specific stuff */
	CRYPTO_set_id_callback((unsigned long (*)())PR_GetCurrentThread);

	/* Set the thread_data_index to non-zero per thread to remove the
	 * SSLeay specic error stuff 
	 */
	PR_NewThreadPrivateIndex(&thread_data_index,\
				mozilla_ssleay_error_distructor);

	SSLeay_add_ssl_algorithms();

	/* hmm... do we want to be able to change this?
	 * make reconfige easier
	 */
	if (ssl_ctx != NULL)
		SSL_CTX_free(ssl_ctx);

	if ((ssl_ctx=SSL_CTX_new(SSLv23_client_method())) == NULL)
		goto err;

	/* TLS is not visible for the moment at least */
	SSL_CTX_set_options(ssl_ctx,SSL_OP_ALL|SSL_OP_NO_TLSv1);
	SSL_CTX_set_cipher_list(ssl_ctx,"DEFAULT");

	/* needs to be fixed */
	SSL_CTX_set_verify(ssl_ctx,SSL_VERIFY_NONE,NULL);

	/* very much needs to be fixed :-) */
	SSL_CTX_load_verify_locations(ssl_ctx,NULL,"../SSLeay/certs");

	/* nothing like a nice choice of options */
	SSL_CTX_set_quiet_shutdown(ssl_ctx,1);
	SSL_CTX_sess_set_cache_size(ssl_ctx,128);
	SSL_CTX_set_session_cache_mode(ssl_ctx,SSL_SESS_CACHE_OFF);

	ssl_window_id_idx=SSL_get_ex_new_index(0,
		"MWContext *window_id",NULL,NULL,NULL);

	ssleay_ssl_identity=PR_GetUniqueIdentity("SSL (SSLeay)");

	if (dbglev)
	  BIO_printf(bio_err,"ssleay.c:SSLeay_init - OKAY\n");

	return(1);
err:

	if (dbglev)
	  BIO_printf(bio_err,"ssleay.c:SSLeay_init - FAILED\n");

	return(0);

	}

void thread_cleanup()
        {
        int i;

	if (dbglev)
	  BIO_printf(bio_err,"ssleay.c:thread_cleanup\n");

	CRYPTO_set_locking_callback(NULL);
	for (i=0; i<CRYPTO_NUM_LOCKS; i++)
		PR_DestroyLock(lock_cs[i]);
	}


int ssleay_enable_ssl(win,sock)
MWContext *win;
PRFileDesc *sock;
	{
	static int init_done_hack=0;
	PRFileDesc *tmp;

	/* make things auto-init so we don't have to go looking
	 * for the 'right' place to hook things for now
	 */
	if (!init_done_hack) {
		SSLeay_init();
		init_done_hack=1;
        }

	/* hook the BIO and the layer together now */
	tmp=ssl_SSLeay_new(win);
	PR_PushIOLayer(sock,PR_GetLayersIdentity(sock),tmp);

	return(1);
	}


